from ximalaya.api.typing import ResponsePaginator, FollowingInfo, UserBasicInfo, UserDetailedInfo, UserSubscription
from ximalaya.client import XimalayaClient


class FollowingResponsePaginator(ResponsePaginator):
    def __next__(self) -> list[FollowingInfo]:
        data = self.client.get(f'{self.path}&page={self.page_num}')['data']

        followings = data['followingsPageInfo']

        if len(followings) == 0:
            raise StopIteration()

        self.page_num += 1

        return followings


class SubscriptionResponsePaginator(ResponsePaginator):
    def __next__(self) -> list[UserSubscription]:
        data = self.client.get(f'{self.path}&page={self.page_num}')['data']

        subs = data['albumsInfo']

        if len(subs) == 0:
            raise StopIteration()

        self.page_num += 1

        return subs


def get_user_followings(client: XimalayaClient, uid: int, page_size: int = 10) -> FollowingResponsePaginator:
    return FollowingResponsePaginator(client=client, path=f'/revision/user/following?uid={uid}&pageSize={page_size}&keyWord=')


def get_user_basic_info(client: XimalayaClient, uid: int) -> UserBasicInfo:
    return client.get(f'/revision/user/basic?uid={uid}')['data']


def get_user_detailed_info(client: XimalayaClient, uid: int) -> UserDetailedInfo:
    return client.get(f'/revision/user?uid={uid}')['data']


def get_user_subscriptions(client: XimalayaClient, uid: int, page_size: int = 10) -> list[UserSubscription]:
    return client.get(f'/revision/user/sub?pageSize={page_size}&keyWord=&uid={uid}')



from abc import ABC
from typing import TypedDict, Iterator

from ximalaya.client import XimalayaClient


class FollowingInfo(TypedDict):
    uid: int
    coverPath: str
    anchorNickName: str
    background: str
    description: str
    url: str
    grade: int
    mvpGrade: int
    gradeType: int
    trackCount: int
    albumCount: int
    followerCount: int
    followingCount: int
    isFollow: bool
    beFollow: bool
    isBlack: bool
    logoType: int
    ptitle: str


class FollowingPageInfo(TypedDict):
    totalCount: int
    followInfoList: list[FollowingInfo]


class FollowingResponse(TypedDict):
    ret: int
    msg: str
    data: dict[{'followingsPageInfo': list[FollowingInfo]}]
    page: int
    pageSize: int
    totalCount: int
    uid: int
    maxCount: int


class SubscriptInfo(TypedDict):
    albumSubscriptValue: int
    url: str


class AlbumInfo(TypedDict):
    albumId: int
    albumPlayCount: int
    albumTrackCount: int
    albumCoverPath: str
    albumTitle: str
    albumUserNickName: str
    anchorId: int
    anchorGrade: int
    mvpGrade: int
    isDeleted: bool
    isPaid: bool
    isFinished: int
    anchorUrl: str
    albumUrl: str
    intro: str
    vipType: int
    logoType: int
    subscriptInfo: SubscriptInfo
    albumSubscript: int


class AlbumsResponse(TypedDict):
    currentUid: int
    total: int
    pageNum: int
    pageSize: int
    albums: list[AlbumInfo]


class PubInfo(TypedDict):
    id: int
    title: str
    subTitle: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchorUrl: str
    anchorNickname: str
    anchorUid: int
    playCount: int
    trackCount: int
    albumUrl: str
    description: str
    vipType: int
    albumSubscript: int


class PubPageInfo(TypedDict):
    totalCount: int
    pubInfoList: list[PubInfo]


class TrackInfo(TypedDict):
    trackId: int
    title: str
    trackUrl: str
    coverPath: str
    createTimeAsString: str
    albumId: int
    albumTitle: str
    albumUrl: str
    anchorUid: int
    anchorUrl: str
    nickname: str
    durationAsString: str
    playCount: int
    showLikeBtn: bool
    isLike: bool
    isPaid: bool
    isRelay: bool
    showDownloadBtn: bool
    showCommentBtn: bool
    showForwardBtn: bool
    isVideo: bool
    videoCover: str
    breakSecond: int
    length: int
    isAlbumShow: bool


class TrackPageInfo(TypedDict):
    totalCount: int
    trackInfoList: list[TrackInfo]


class SubscriptionPageInfo(TypedDict):
    privateSub: bool
    totalCount: int
    subscribeInfoList: list


class FansInfo(TypedDict):
    uid: int
    coverPath: str
    anchorNickName: str
    background: str
    url: str
    grade: int
    mvpGrade: int
    gradeType: int
    trackCount: int
    albumCount: int
    followerCount: int
    followingCount: int
    isFollow: bool
    beFollow: bool
    isBlack: bool
    logoType: int


class FansPageInfo(TypedDict):
    totalCount: int
    fansInfoList: list[FansInfo]


class UserDetailedInfo(TypedDict):
    uid: int
    pubPageInfo: PubPageInfo
    trackPageInfo: TrackPageInfo
    subscriptionPageInfo: SubscriptionPageInfo
    followingPageInfo: FollowingPageInfo


class UserBasicInfo(TypedDict):
    uid: int
    nickName: str
    cover: str
    background: str
    isVip: bool
    constellationType: int
    personalSignature: str
    personalDescription: str
    fansCount: int
    gender: int
    birthMonth: int
    birthDay: int
    province: str
    city: str
    anchorGrade: int
    mvpGrade: int
    anchorGradeType: int
    isMusician: bool
    anchorUrl: str
    logoType: int
    followingCount: int
    tracksCount: int
    albumsCount: int
    albumCountReal: int
    userCompany: str
    qualificationGuideInfos: list[str]


class UserSubscription(TypedDict):
    id: int
    title: str
    subTitle: str
    description: str
    coverPath: str
    isFinished: bool
    isPaid: bool
    anchor: dict[{'anchorUrl': str, 'anchorNickName': str, 'anchorUid': int, 'anchorCoverPath': str, 'logoType': int}]
    playCount: int
    trackCount: int
    albumUrl: str
    albumStatus: int
    lastUptrackAt: int
    lastUptrackAtStr: str
    serialState: int
    isTop: bool
    categoryCode: str
    categoryTitle: str
    lastUptrackUrl: str
    lastUptrackTitle: str
    vipType: int
    albumSubscript: int
    albumScore: str


class UserSubscriptionResponse(TypedDict):
    ret: int
    msg: str
    data: dict[{'albumsInfo': list[UserSubscription]}]
    privateSub: bool
    page: int
    pageSize: int
    totalCount: int
    uid: int
    maxCount: int


class ResponsePaginator(ABC, Iterator):
    client: XimalayaClient
    path: str
    page_num: int

    def __init__(self, client: XimalayaClient, path: str, page_num: int = 1):
        self.client = client
        self.path = path
        self.page_num = page_num



import json
import urllib.request


class XimalayaClient:
    def __init__(self, host: str = 'www.ximalaya.com', headers: dict = None):
        self.host = host

        self.headers = {} if headers is None else headers

        if 'User-Agent' not in self.headers:
            self.headers['User-Agent'] = ''

    def get(self, path: str, method: str = 'GET'):
        url = f'https://{self.host}/{path.lstrip("/")}'

        req = urllib.request.Request(url, headers=self.headers, method=method)

        with urllib.request.urlopen(req) as fp:
            return json.loads(fp.read().decode('utf-8'))



from ximalaya.api.typing import ResponsePaginator, AlbumInfo
from ximalaya.client import XimalayaClient


class AlbumsResponsePagination(ResponsePaginator):
    def __next__(self) -> list[AlbumInfo]:
        data = self.client.get(f'{self.path}&pageNum={self.page_num}')['data']

        albums = data['albums']

        if len(albums) == 0 or data['total'] == 0:
            raise StopIteration()

        self.page_num += 1

        return albums


def get_category_albums(client: XimalayaClient, category_id: int, page_size: int = 56, sort_by: int = 1) -> AlbumsResponsePagination:
    return AlbumsResponsePagination(client=client, path=f'/revision/category/v2/albums?pageSize={page_size}&sort={sort_by}&categoryId={category_id}')
